const express = require("express");
const tmpfs    = require('temporary');


const { spawn } = require('child_process');

module.exports = app => {

    function getDOI(script_name,doi,output_json_file,res) {

	var output_json_filename = output_json_file.path;
	
	var dataToSend;
	
	// Spawn new child process to call the python script
	console.log("Running python3 " + script_name + " " + doi + " " + output_json_filename);
	const python = spawn('python3', [script_name, doi, output_json_filename]);
	
	// Collect data from script
	python.stdout.on('data', function (data) {
	    //console.log('Pipe data from python script ...');
	    dataToSend = data.toString();
	});
	
	// In close event we are sure that stream from child process is closed
	python.on('close', (code) => {
	    console.log(`child process closed, with exit value: ${code}`);
	    
	    //res.send(dataToSend)
	    //res.sendFile(output_json_filename, { root: __dirname }, function(err) {
	    res.sendFile(output_json_filename, function(err) {
		if (err) {
		    console.error("Error: Failed to send file '" + output_json_filename + "'");
		    console.error(err);
		    res.status(404);
		    //res.status(err.status);
		    res.end();
		}
		else {
		    console.log("Away to remove: '" + output_json_filename + "'");
		    output_json_file.unlinkSync();
		}
	    });
	});	
    }
    
    function OACoreGetDOI(doi,res)
    {
	var unique_tmp_file = new tmpfs.File("oacore-output");
	console.log("unique tmp file = " + unique_tmp_file.path);
	
	getDOI("doi-stinky/oacore-get-doi.py", doi, unique_tmp_file, res);
	
	//getDOI("doi-stinky/oacore-get-doi.py", doi, "oacore-output.json", res);
    }

    function CrossRefGetDOI(doi,res)
    {
	var unique_tmp_file = new tmpfs.File("crossref-output");
	getDOI("doi-stinky/crossref-get-doi.py", doi, unique_tmp_file, res);

	//getDOI("doi-stinky/crossref-get-doi.py", doi, "crossref-output.json", res);
    }
    

    
    let router = express.Router();

    // General logging 
    router.use(function(req,res,next) {
        console.log("/" + req.method);
        next();
    });

    // API Routing Rules

    router.get("/",function(req,res) {
        res.json({"message" : "A Gateway API for accessing a range of DOI-based APIs provided by others"});
    });


    router.get("/get-doi", (req, res) => {
	var oacoreDOI   = req.query.coreDOI;
	var crossrefDOI = req.query.crossrefDOI;

	if (oacoreDOI) {
	    OACoreGetDOI(oacoreDOI,res);
	}
	else if (crossrefDOI) {
	    CrossRefGetDOI(crossrefDOI,res);
	}
	
    });
    
    app.use("/api",router);    
}

